/* libfir/src/libfir++/100_Filter.h
 * 
 *  Copyright: ©2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



/*! \defgroup libfirpp_fir Basic FIR filtering

\ingroup libfirpp

*/



/*! \brief %FIR filter symmetry.

\ingroup libfirpp_fir

The library extensively takes advantage of the symmetry that is often present
in %FIR filter coefficient sets.

\note The numeric constants match those used in \ref libfir, so it is possible
to pass them directly between the libraries as required.

*/
enum Symmetry {
    /*! \brief No symmetry. */
    SymmetryNone,

    /*! \brief Odd symmetry (odd total number of points). */
    SymmetryOdd,

    /*! \brief Even symmetry (even total number of points). */
    SymmetryEven,
};



/*! \brief Basic %FIR filter object.

\ingroup libfirpp_fir

*/
class Filter {
public:
    /*! \brief Construct new filter.

    \param ncoeff Number of coefficients in array \a coeff.
    \param coeff Pointer to array of %FIR filter coefficients.
    \param symm Whether coefficients are symmetric or not.

    Builds a new filter. The number of coefficients \a ncoeff should reflect the
    number of elements in the array \a coeff. If the filter is symmetric, the
    actual %FIR filter itself will consist of \a 2*ncoeff (even symmetry) or
    \a 2*ncoeff-1 (odd symmetry) points, as the symmetrical coefficients are
    not passed.

    */
    Filter(int ncoeff, const double* coeff, Symmetry symm = SymmetryNone);



    /*! \brief Construct filter by copying coefficients from another instance.

    \param other Filter to copy coefficients from.

    Constructs a new filter object which copies the coefficients (but not the
    state, i.e. previous sample values) from the filter object \a other.

    */
    Filter(const Filter& other);



    /*! \brief Construct new filter.

    \param coeff Array of coefficients.
    \param symm Whether coefficients are symmetric or not.

    Builds a new filter. If the filter coefficient set is symmetric, then
    \a coeff should only contain the non-repeated elements.

    */
    Filter(const std::vector<double>& coeff, Symmetry symm = SymmetryNone);



    /*! \brief Take ownership of underlying C filter object.

    \param fi Filter.

    This constructor may be used to build a filter object using an underlying C
    library filter object. The new instance will take ownership of \a fi and
    free it when the instance is deleted.

    */
    explicit Filter(struct fir_filter_t* fi);



    /*! \brief Destructor (frees underlying memory). */
    virtual ~Filter();



    /*! \brief %Filter a sample.

    \param x Input sample.
    \returns Filter output.

    Filters a sample. If this is the very first sample passed to the instance
    after construction, it will be taken as the steady-state DC input and the
    filter primed appropriately.

    */
    double f(double x);



    /*! \brief Query filter's symmetry.

    \returns Symmetry of filter.

    */
    Symmetry symmetry() const;


    /*! \brief Query number of coefficients in filter.

    \returns Number of coefficients.

    This function returns the number of coefficients. It ignores symmetry, so it
    returns the total number of coefficients regardless of whether symmetry has
    allowed us to store only half of them.

    */
    int ncoeff() const;



    /*! \brief Retrieve value of coefficient from filter.

    \param idx Index of coefficient (0 ≤ \a idx &lt; \ref ncoeff()).
    \returns Value of coefficient.

    This function retrieves the coefficient at \a idx. This works even for
    coefficients that are not actually stored due to symmetry.

    */
    double coeff(int idx) const;



    /*! \brief Evaluate response of filter at given frequency.

    \param freq Frequency as a fraction of sampling rate.
    \returns Complex response of filter at given frequency.

    Performs Z-domain analysis of the filter to determine its complex response
    (phase and magnitude) to a signal of the given frequency \a freq, which is
    expressed as a fraction of the sampling rate being used with the filter.

    */
    std::complex<double> response(double freq) const;



private:
    struct fir_filter_t* fi_;

    // disallow assignment
    Filter& operator=(const Filter& other);
};



/* options for text editors
vim: expandtab:ts=4:sw=4:syntax=cpp.doxygen
*/
