/* libfir/src/libfir/200_common_filters.c
 *
 *  Copyright: ©2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



static double
window_rectangle(int n, int N)
{
    return 1.0;
}



static double
window_blackman(int n, int N)
{
    double v = 2 * M_PI * n;
    v /= N;
    return 0.42 - 0.5 * cos(v) + 0.08 * cos(2 * v);
}



static struct fir_filter_t*
fir_windowed_lowpass(int npoints, double corner, double gain,
    double (*window)(int n, int N))
{
    double* coeff, t, sum = 0;
    int i, spoints;

    if(npoints < 1) {
        errno = EINVAL;
        return 0;
    }

    /* find number of points after taking symmetry into account */
    spoints = (npoints + 1) / 2;
    coeff = alloca(sizeof(double) * spoints);

    /* only fill out for t ≤ 0 */
    for(i = 0; i < spoints; ++i) {
        if((i * 2 + 1) == npoints) {
            coeff[i] = window(i, npoints) * 1;
            sum += coeff[i]; /* not 2, as we don't repeat this coeff */
        } else {
            t = i + 0.5 - npoints * 0.5;
            t *= corner * 2 * M_PI;
            coeff[i] = window(i, npoints) * sin(t) / t;
            sum += coeff[i] * 2;
        }
    }

    /* normalise gain */
    gain /= sum;
    for(i = 0; i < spoints; ++i) {
        coeff[i] *= gain;
    }

    return fir_filter_new(spoints, coeff,
        (npoints & 1) ? fir_symmetry_odd : fir_symmetry_even);
}



struct fir_filter_t*
fir_sinc_lowpass(int npoints, double corner, double gain)
{
    return fir_windowed_lowpass(npoints, corner, gain, window_rectangle);
}



struct fir_filter_t*
fir_blackman_lowpass(int npoints, double corner, double gain)
{
    return fir_windowed_lowpass(npoints, corner, gain, window_blackman);
}



struct fir_filter_t*
fir_lanczos_lowpass(int npoints, double corner, double gain, int a)
{
    double* coeff, t, sum = 0;
    int i, spoints;

    if(npoints < 1) {
        errno = EINVAL;
        return 0;
    }

    /* find number of points after taking symmetry into account */
    spoints = (npoints + 1) / 2;
    coeff = alloca(sizeof(double) * spoints);

    /* only fill out for t ≤ 0 */
    for(i = 0; i < spoints; ++i) {
        if((i * 2 + 1) == npoints) {
            coeff[i] = 1;
            sum += 1; /* not 2, as we don't repeat this coeff */
        } else {
            t = i + 0.5 - npoints * 0.5;
            t *= corner * 2 * M_PI;
            coeff[i] = (sin(t) / t) * sin(t / a) / (t / a);
            sum += coeff[i] * 2;
        }
    }

    /* normalise gain */
    gain /= sum;
    for(i = 0; i < spoints; ++i) {
        coeff[i] *= gain;
    }

    return fir_filter_new(spoints, coeff,
        (npoints & 1) ? fir_symmetry_odd : fir_symmetry_even);
}



/* options for text editors
vim: expandtab:ts=4:sw=4
*/
